#define SCOREFILE ".cvscores"

#include <sys/types.h>
#include <sys/time.h>
#include <errno.h>
#include <math.h>
#include "global.h"
#include "score.h"

/***************************************************************************\
|*									   *|
|*  I'm gonna define the standard movement keys here at the top of the	   *|
|*  file so that you can see exactly what I've done.  Other than what	   *|
|*  follows here, and the getenv() bit in main(), the only other change	   *|
|*  is to the key-recognition near the bottom of main() - Mirk		   *|
|*									   *|
\***************************************************************************/

char _up_key = 'q';
char _down_key = 'a';
char _left_key = ',';
char _right_key = '.';
char _fire_key = ' ';
char _select_key = 's';
char _auto_destruct_key = '*';
char _cheat_key = '%';

/*-------------------------------------------------------------------------*/

extern time_t timeofday;

int monsterstate1, monsterstate2, monsterstate3;
int monsterx[MAXMONSTERS], monstery[MAXMONSTERS], monsterc[MAXMONSTERS];
int stage, stageend, hashcount;
int wpoints, score, shield, x, y, speedtime, hit;

float missx[MAXMISSILES], missy[MAXMISSILES], missvx[MAXMISSILES],
      missvy[MAXMISSILES];
int missf[MAXMISSILES];

/****************************************************************************\
 FUNCTION: safeopenscores.

\****************************************************************************/

int safeopenscores ()
{
    int fd;

    switch ( fd = openscore ( SCOREFILE, TIMEOUT, OPENSCORE ) ) {
    case -3:
	perror ( SCOREFILE );
	exit ( 1 );
	break;
    case -2:
	printf ( "Creating score file: %s\n", SCOREFILE );
	switch ( newscore ( SCOREFILE, TIMEOUT ) ) {
	case -2:
	    printf ( "Score file doesnt really exist?\n" );
	    perror ( SCOREFILE );
	    exit ( 2 );
	    break;
	case -1:
	    printf ( "Score file does exist but is locked!\n" );
	    perror ( SCOREFILE );
	    exit ( 3 );
	    break;
	}
	exit ( 0 );
	break;
    case -1:
	printf ( "Score file does exist but is locked!\n" );
	perror ( SCOREFILE );
	exit ( 4 );
	break;
    }
    return ( fd );
}

/****************************************************************************\
 FUNCTION: printscores.

\****************************************************************************/

void printscores ( option )
    char *option;
{
    int num, dummy, fd, i;

    num = width ( option+2, DEFSCORES, & dummy );
    if ( num < 1 || num > STSIZE ) {
	printf ( "Phlegm off - thats a silly number!\n" );
	exit ( 1 );
    }
    fd = safeopenscores ();
    (void) printf ( ".-----.---------------------------.---------.--------------------------------.\n");
    (void) printf ( "| No. | Name                      | Score   | Level                          |\n");
    (void) printf ( ">-----+---------------------------+---------+--------------------------------<\n");
    for ( i = 1; i <= num; i++ )
	(void) printf ( "| %3d %s\n",
		       i,
		       getscore ( fd, i, "| %-25n | %7s | %-30i |" ) );

    (void) printf ( "`-----^---------------------------^---------^--------------------------------'\n");
    (void) closescore ( fd );
}

/****************************************************************************\
 PROCEDURE: dohiscore.

\****************************************************************************/

void dohiscore ( score, level )
    int score;
    int level;
{
    int fd, rank;
    char *name, *getenv ();

    fd = safeopenscores ();
    if ( ( name = getenv ( "VOMITNAME" ) ) == NULL ) {
	if ( ( name = getenv ( "NAME" ) ) == NULL )
	    name = "Vomit-novice";
    }
    switch ( ( rank = addscore ( fd, name, score, level, stagename[level-1] ) ) ) {
    case -1:
	perror ( "Cheese up in score file! ");
	exit ( 1 );
	break;
    case 0:
	break;
    default:
	(void) move ( 10, 22 );
	(void) printw ( "You rank #%d in the hiscore table!\n", rank );
	(void) refresh ();
	(void) usleep ( 500000 );
	break;
    }
    (void) closescore ( fd );
}

/****************************************************************************\
 FUNCTION: pollkeyboard.
 Finds out whether there are any characters to be read in from stdin.
 It returns the following values:-
       0  : No char to be read in the past (sec+usec/10^6).
       1  : There are chars to be read.
      -1  : Phlegm up!

\****************************************************************************/

int pollkeyboard ( sec, usec )
    int sec;
    int usec;
{

    fd_set rd, wr, xp;
    struct timeval timeout;

    timeout.tv_sec = sec;
    timeout.tv_usec = usec;

    FD_ZERO ( &rd );
    FD_ZERO ( &wr );
    FD_ZERO ( &xp );

    FD_SET ( 0, &rd );

    switch ( select ( 32, &rd, &wr, &xp, &timeout ) ) {
    case -1:
	return ( -1 );
    case 0:
	return ( 0 );
    default:
	if ( FD_ISSET ( 0, &rd ) )
	    return ( 1 );
	else
	    return ( 0 );
    }
}

/***************************************************************************\
|*									   *|
|*  The function flush_keybord() checks to see whether there are any	   *|
|*  keystrokes waiting to be read, and if so reads them and throws	   *|
|*  them away.	Surprising how complex this needs to be, but the only	   *|
|*  alternative I can think of is to ioctl() stdin to be non-blocking,	   *|
|*  read() until it returns 0, then ioctl() it back, which is scarcely	   *|
|*  an improvement. Thanx to Unkle Chicken Mirk    			   *|
|*									   *|
\***************************************************************************/

void flush_keyboard ()
{
  fd_set fds;			/* Will contain only stdin */
  struct timeval timeout;	/* Will contain a zero timeout (poll) */
  static char buffer[LINELEN];	/* Will be used as the buffer to read(2) */
  int status;			/* Contains select(2)'s return value */

  while (1) {
    FD_ZERO (&fds);
    FD_SET (0, &fds);
    timeout.tv_sec = 0L;
    timeout.tv_usec = 0L;
    if (((status = select (1, &fds, (fd_set*) NULL, (fd_set*) NULL,
			   &timeout)) == -1) && (errno != EINTR))
      exit ( 1 );
    if ((status == 0) || (status == -1))
      break;			/* Can only be -1 if errno == EINTR */
    (void) read (0, buffer, LINELEN);
  }
}



/****************************************************************************\
 PROCEDURE: vaddch.
 Does an addch only if it is on screen.

\****************************************************************************/

void vaddch( a, b, vchar )
    int a;
    int b;
    char vchar;
{
    if ( (b>=1) && (b<22) && (a>0) && (a<80) ) {
	(void) move ( b, a );
	(void) addch ( vchar );
    }
}

/****************************************************************************\
 PROCEDURE: update.
 Refreshes the screen and get the cursor out of the frogging way.

 \***************************************************************************/

void update ()
{
    (void) move ( 0, 0 );
/*    (void) addstr ( " " );*/
    (void) refresh ();
}

/****************************************************************************\
 FUNCTION: checkoption.
 Receives a string, and checks to see if string has been passed as an argument
 to the program.
 Returns position if found, 0 if not.

\****************************************************************************/

int checkoption( numoptions, option, checkoption )
    int numoptions;
    char *option[];
    char *checkoption;
{
    int i;

    for ( i=1 ; i<numoptions ; i++ )
	if ( ! strncmp ( option[i] , checkoption, strlen ( checkoption ) ) )
	    return ( i );
    return ( 0 );
}

/****************************************************************************\
 FUNCTION: testing.
 Returns level to start on-1, and if required ask for a password.
 Also chucks non-validated testers off if the load average is 2 flemin hi!
\****************************************************************************/

int testing ( argc, argv )
    int argc;
    char **argv;
{
    int dummy, level, index = checkoption ( argc, argv, "-test" );
    char *password;

#ifdef GAMESOK
    int lth = 0;
    lth = system ( "/usr/local/gamesok -s" );
    if ( ! index && ! lth )
	return ( 0 );
#endif /*GAMESOK*/

    if ( index > 0 ) {
	level = width ( argv[index]+5, NUMSTAGES, & dummy ) - 1;
	password = (char *) getpass ("Password:");
	if (strcmp ( (char *) crypt ( password, TESTPASSWORD ),
		                                TESTPASSWORD ) == 0)
	    return ( level );
	else {
	    (void) printf ( "Phlegm off! You can't cheat now!\n" );
	    exit (1);
	}
    }

#ifdef GAMESOK
    if ( lth ) {
	printf ("Load average too high! Go play tetris vomit head.\n");
	exit (1);
    }
#endif /*GAMESOK*/
}

/****************************************************************************\
 FUNCTION: rnd.  Returns a random integer in the range [0,10000].

\****************************************************************************/

int rnd()
{
    return ( (int) ( random() / RANDDIV ) );
}


/****************************************************************************\
 FUNCTION: moveship Moves the ship on the screen and updates.
 Returns the following values:-
           0 : Everything ok.
           1 : Ship has collided and died.

NB. Do not call with cx != 0 && cy != 0 !!!!! It will phlegm up./
\****************************************************************************/

int moveship ( sx, sy, cx, cy )
    int sx;
    int sy;
    int cx;
    int cy;
{
    int t;
    char collide;

    (void) move ( sy, sx );
    (void) addstr ( "    " );

    for ( t=0 ; t<4 ; t++ ) {
	(void) move ( sy+cy, sx+cx+t );
	if ( ( collide = inch() ) != ' ' )
	    break;
    }
    (void) move ( sy+cy, sx+cx);
    (void) addstr ( ship );
/*    update(); */
    switch ( collide ) {
    case '#':
	if ( hashcount < NUMWEAPONS ) {
	    wpoints++;
	    hashcount++;
	}
	/* Cant have equal to NUMWEAPONS ! */
	if ( wpoints > NUMWEAPONS )
	    wpoints = 0;
	score += 33;
	printinfo();
	return ( 0 );
	break;

    case ' ':
	return ( 0 );
	break;
    default:
	return ( 1 );
    }
}

/****************************************************************************\
 PROCEDURE: die Blow the ship all over the shop.

\****************************************************************************/

void die ( dx, dy )
    int dx;
    int dy;
{
    int count = 0, t;
    float bx[16], by[16], vx[16], vy[16];
    char bits[16];
    int flag[16];

    (void) move ( dy, dx );
    (void) addstr ( " " );
    (void) strcpy ( bits , ">C=-++--++--++--" );
    for ( t=0 ; t<16 ; t++ ) {
	flag[t] = 0;
	bx[t] = dx;
 	by[t] = dy;
 	vx[t] = ( (float) (rnd())/3700.0 + 0.06) ;
 	if ( rnd() < 5000 ) vx[t] = -vx[t];
 	vy[t] = ( (float) (rnd())/10005.0 + 0.02);
 	if ( rnd() < 5000 ) vy[t] = -vy[t];
    }
    while ( count < 16 ) {
	for ( t=0 ; t<16 ; t++ ) {
	    if ( flag[t] == 0 ) {
		(void) move ( (int) (by[t]), (int) (bx[t]) );
		(void) addch ( ' ' );

 		bx[t] = bx[t] + vx[t];
 		if ( ( bx[t] < 1 ) || ( bx[t] > 75 ) ) {
		    flag[t] = 1;
		    count++;
		}
		by[t] = by[t] + vy[t];
		vy[t] += 0.04;
		if ( ( by[t] < 3 ) || ( by[t] > 21 ) ) {
		    flag[t] = 1;
		    count++;
		}
		(void) move ( (int) (by[t]), (int) (bx[t]) );
		(void) addch ( bits[t] );
	    }
	}
	update();
	(void) usleep ( 50000 );
    }
}

/***************************************************************************\
|*									   *|
|*  All the x- and y-coordinates and velocities here are long integers,	   *|
|*  holding values in multiples of a 1/SCALEth of a character-space.	   *|
|*									   *|
\***************************************************************************/

void explode (y_base, x_base, size)
  int y_base;
  int x_base;
  int size;
{
  static char debris[EXP_STATES+1] = "|/-\\";
  static long int x[EXP_SIZE];	/* X-ordinate of piece of debris */
  static long int y[EXP_SIZE];	/* Y-ordinate of piece of debris */
  static long int s[EXP_SIZE];	/* Status of piece of debris */
  static long int dx[EXP_SIZE];	/* Rate of change of X-ordinate */
  static long int dy[EXP_SIZE];	/* Rate of change of Y-ordinate */
  static long int ds[EXP_SIZE];	/* Rate of change of status */
  static int dead[EXP_SIZE];	/* Set to 1 when debris dies */

  int i, j, no_dead = 0;	/* General counters/indices */

  if (size > EXP_SIZE)
    size = EXP_SIZE;		/* Don't iterate past end of arrays */
  move (y_base, x_base);	/* To get the background char */

  for (i = 0; i < size; i++) {	/* Initialise each piece of debris */
    x[i] = x_base*SCALE;	/* Set their initial positions ... */
    y[i] = y_base*SCALE;	/* at the origin coordinates */
    s[i] = random ()%(EXP_STATES*SCALE);
    dx[i] = random ()%SCALE;
    dy[i] = random ()%SCALE;
    ds[i] = random ()%SCALE;
    dead[i] = 0;

    if (random()%100 < x_base+10) /* Go randomly either way, */
      dx[i] = -dx[i];		/* Biased by position across screen */
    if (random()%20 < y_base+3)	/* Go randomly either way, biased by */
      dy[i] = -dy[i];		/* depth down screen and gravity */
  }

  while (no_dead < size) {
    for (i = 0; i < size; i++) {
      if (dead[i] == 0) {
	vaddch ( x[i]/SCALE, y[i]/SCALE, ' ' );
	x[i] += dx[i];		/* Flying left and right */
	y[i] += dy[i];		/* Flying up and down */
	dy[i] += 4;		/* For the gravity effect */
	if ((x[i] < 0) || (x[i] > 79*SCALE) ||
	    (y[i] < 0) || (y[i] > 23*SCALE)) {
	  dead[i] = 1;
	  no_dead++;
	  break;
	}

	s[i] += ds[i];		/* For the rotation effect */
	if (s[i] >= EXP_STATES*SCALE)
	  s[i] -= EXP_STATES*SCALE;
	else
	  if (s[i] < 0)
	    s[i] += EXP_STATES*SCALE;

	vaddch ( x[i]/SCALE, y[i]/SCALE, debris[s[i]/SCALE]);
      }
    }

    update ();
    usleep (10000);
  }
}

/****************************************************************************\
FUNCTION: checkship.
  Checks to see if the ship is unharmed by recent activities.
  Ship can be overlayed only by a space.  Returns the following values:-
                   0 : Everything ok and hoopy.
                   1 : Ship has been hit by something

\****************************************************************************/

int checkship()
{
    int t;
    char check;

    for ( t=0 ; t<4 ; t++ ) {
	(void) move ( y, x+t );
 	check = inch();
 	if ( ( check != ship[t] ) && ( check != ' ' ) ) return( 1 );
	(void) addch( ship[t] );
    }
    return ( 0 );
}


/****************************************************************************\
PROCEDURE: dissolve.
 Dissolves the rectangle defined as ( x in [a,b], y in [c,d] ),
 and then clear the block completely.

\****************************************************************************/

void dissolve ( a, b, c, d )
    int a, b, c, d;
{

    int xrange = b - a ;
    int yrange = d - c ;
    int u, v, t;

    for ( t=0 ; t<800 ; t++ ) {
	u = a + (int) ( rnd() * xrange / 10000 );
	v = c + (int) ( rnd() * yrange / 10000 );
 	(void) move ( v, u );
	(void) addch ( ' ' );
 	update();
    }
    for ( u=a ; u<b ; u++ ) {
	for ( v=c ; v<d ; v++ ) {
	    (void) move ( v, u );
	    (void) addch ( ' ' );
	}
    }
    update();
}


/****************************************************************************\
PROCEDURE: printinfo.

\****************************************************************************/

void printinfo ()
{
    (void) move ( 22, 1 );
    (void) printw ( " Score: %i " , score );
    (void) move ( 22, 20 );
    (void) printw ( "Stage: %i " , stage );
    (void) move ( 22, 31 );
    (void) printw ( "[%i]  " , shield );
    (void) printw ( "<%i>  " , speedtime );
    (void) printw ( "Select >%s<         " , weaponname[wpoints] );
}

/****************************************************************************\
PROCEDURE: introduction.
Do all the carp.

\****************************************************************************/

void introduction()
{
    int i, j;
    float x[20], xv[20];
    char carrots[20];

    (void) strcpy ( carrots , "@#$%&*@#$%&*@#$%&*@#" );

    for ( i=14 ; i>=0 ; i-- ) {
	for ( j=i ; j<14 ; j++ ) {
	    (void) move ( j-i, 4 );
	    (void) addstr ( title[j] );
	}
 	update();
    }

    for ( i=0 ; i<20 ; i++ ) {
	x[i] = 22;
 	xv[i] = ( (float) rnd() - 5000 ) / 2000;
    }

    for ( j=11 ; j<20 ; j++ ) {
	for ( i=0 ; i<20 ; i++ ) {
	    x[i] = x[i] + xv[i];
	    (void) move ( j, (int) (x[i]) );
	    (void) addch ( carrots[i] );
 	}
	update();
    }

    for ( i=0 ; i<14 ; i++ ) {
 	for ( j=i ; j<14 ; j++ ) {
	    (void) move ( j-i, 4 );
	    (void) addstr ( title[j] );
	}
	update();
    }
    (void) clear();
}

/****************************************************************************\
 PROCEDURE: initialisemonsters.
 Set the appropiate monsterx, monstery values.

\****************************************************************************/

void initialisemonsters()
{
    int i, start;

    monsterstate1 = monsterstate2 = monsterstate3 = 0;
    for ( i=0 ; i<MAXMISSILES ; i++ ) missf[i] = 0;
    switch ( stage ) {
    case 1:
	monsterstate2 = 1;
	monsterx[0] = 45;
	monstery[0] = 8;
	break;

    case 2:
	for ( i=0 ; i<NUMVOMIT ; i++ ) {
	    start = rnd () / 100;
	    monsterx[i] = start + 20 ;
	    monstery[i] = start / 5 + 1;
	}
	break;

    case 4:
	for ( i=0 ; i<NUMCHICKS ; i++ ) {
	    monsterx[i] = 73 - rnd()/250;
	    monstery[i] = (rnd() / 500) + 1;
	}
	break;
    case 5:
	for ( i=0 ; i<NUMTRIBBLES ; i++ ) {
	    monsterx[i] = 73 - rnd()/250;
	    monstery[i] = (rnd() / 500) + 1;
	}
	break;
    case 7:
	missf[0] = 0;
	monsterstate3 = MIRKALIENSPEED*15 -4;
	monsterstate2 = 1;
	monstery[0] = 16;
	monsterx[0] = 34;
	/* initialise the random tongue */
	monsterc[0] = '_';
	for ( i=1 ; i<20 ; i++ ) {
	    monsterx[i] = monsterx[i-1] - 1;
	    monstery[i] = monstery[i-1];
	    monsterc[i] = '_' ;
	    if ( rnd() < 5000 ) {
		monsterc[i-1] = '\\';
		monstery[i]--;
	    }
	}
	break;
    case 11:
	for ( i=0 ; i<NUMPLANTS ; i++ ) {
	    monsterx[i] = 25 + 16*i + rnd() / 900;
	    monstery[i] = 17;
	    monsterc[i] = 0;
	}
	monsterstate1 = -1;
	break;
    case 13:
	for ( i=0 ; i<4 ; i++ ) {
	    Harvship[0][10][3+i] = '|';
	    Harvship[1][10][4+i] = '|';
	}
	break;
    case 9:
	for ( i=0 ; i<NUMSALVOS ; i++ ) {
	    monsterx[i] = 77 - rnd()/250;
	    monstery[i] = 1 + rnd() / 500;
	}
	break;
    case 12:
	for ( i=0 ; i<NUMVOMIT-2 ; i++ ) {
	    start = rnd () / 100;
	    monsterx[i] = start + 20 ;
	    monstery[i] = start / 5 + 1;
	}
	break;
    case 8:
	monsterx[0] = 50;
	monstery[0] = 7;
	break;
    default:
	break;

    }
}

/****************************************************************************\
 PROCEDURE: stagefinish.
 Fancy twiddly bit at the end.

\****************************************************************************/

void stagefinish()
{
    int i;

    for ( i=1 ; i<22 ; i++ ) {
	(void) move ( i, 1 );
	(void) addstr ( "@#$%&*@#$%&*@#$%&*@#$%&*@#$%&*@#$%&*@#$%&*@#$%&*@#$%&*@#$%&*@#$%&*@#$%&*@#$%&*" );
	update();
	(void) usleep ( 9000 );
    }
}

/****************************************************************************\
PROCEDURE: stagechange.
Make all the changes between stages.

\****************************************************************************/

void stagechange()
{
    int i;

    for ( i=1 ; i<22 ; i++ ) {
	(void) move ( i, 1 );
	(void) addstr ( "                                                                              " );
    }
    update();

    x = 5;
    y = 12;
    hashcount = 0;
    (void) move ( y, x );
    (void) addstr ( ship );

    printinfo();
    for ( i=0 ; i<MAXMISSILES ; i++ ) missf[i] = 0;
    stageend = 0;
    for ( i=0 ; i<30 ; i++ ) {
	(void) move ( 12, 25+i );
	(void) addch ( stagename[stage][i] );
	update();
	(void) usleep ( 60000 );
    }
    (void) usleep ( 100000 );
    (void) move ( 12, 25 );
    (void) addstr ( "                              " );
    stage++;
    if ( stage > NUMSTAGES ) {
	stage = 1;
	hit = 1;
	shield = 0;
    }
    initialisemonsters();
    flush_keyboard();
}


/****************************************************************************\
 PROCEDURE: shieldfire
 Fires the life saving hoopy shield.

\****************************************************************************/

void shieldfire ( sx, sy )
    int sx, sy;
{
    int t, a, b;
    char shcr[2];

    (void) strcpy ( shcr , "# " );

    for ( b=0 ; b<6 ; b++ ) {
	a = ( b % 2 );
 	for ( t=0 ; t<6 ; t++ ) {
	    (void) vaddch ( sx+t+1, sy-2+(int) (0.45*t), shcr[a] );
	    (void) vaddch ( sx+t+1, sy+2-(int) (0.45*t), shcr[a] );
	    update();
 	}
    }
    (void) move ( y, x );
    (void) addstr ( ship );
    printinfo();
}

/****************************************************************************\
 PROCEDURE: bitbomb.
 Fires the hoopy smart bomb.

\****************************************************************************/

void bitbomb( bx, by )
    int bx, by;
{
    int sbx[16], sby[16], sbxv[16], sbyv[16], sbf[16], t, count;
    double angle;
    char check;

    for ( t=0; t<16 ; t++ ) {
	angle = PI * ((double) t) / 8.0 + 0.4;
	sbx[t] = (int) (((double) bx)+2.0+4.0*cos(angle));
	sby[t] = (int) (((double) by)+0.5+2.0*sin(angle));
	sbxv[t] = (int) ( 4*cos(angle) );
	sbyv[t] = (int) ( 2*sin(angle) );
	sbf[t] = 1;
	(void) vaddch ( sbx[t], sby[t], 'O' );
    }
    update();

    count = 0;

    while ( count < 16 ) {
	for ( t=0 ; t<16 ; t++ ) {
	    if ( sbf[t] == 1 ) {
		(void) vaddch ( sbx[t], sby[t], ' ' );
		sbx[t] += sbxv[t];
		sby[t] += sbyv[t];
		(void) move ( sby[t], sbx[t] );
		if ( ( check = inch() ) != ' ' ) {
		    laserhit ( check, sbx[t], sby[t] );
		    sbf[t] = 0;
		    count++;
		}
		if ( sbf[t]&&(sbx[t]<2||sbx[t]>79||sby[t]<1||sby[t]>21)) {
		    sbf[t] = 0;
		    count++;
		}
		if ( sbf[t] == 1) (void) vaddch ( sbx[t], sby[t], 'O' );
	    }
	}
	update ();
    }
}

/****************************************************************************\
 PROCEDURE: laser.
 Draws the hoopy laser beam and updates the screen.

\****************************************************************************/

void laser ( fx, fy, type )
    int fx, fy, type;
{
    int t, j;
    char check, laserbits[2];

    (void) strcpy ( laserbits , "~=" );
    for ( t=fx+5 ; t<80 ; t++ ) {
	(void) move ( fy, t );
 	check = inch();
 	if ( check != ' ' ) {
	    for ( j=0 ; j<=type ; j++ ) laserhit ( check, t, fy );
	    break;
 	}
	(void) addch ( laserbits[type] );
    }
    update();
    (void) usleep ( 10000 );

    for ( t=fx+5 ; t<80 ; t++ ) {
	(void) move ( fy, t );
 	check = inch();
 	if ( check != laserbits[type] ) break;
  	(void) addch ( ' ' );
    }

    score -= 4;
}

/****************************************************************************\
 PROCEDURE: hoopybeam.
 Draw the wide beam.

\****************************************************************************/

void hoopybeam ( fx, fy )
    int fx, fy;
{
    int t, i;
    char check;

    for ( i=-1 ; i<=1 ; i++ ) {
	for ( t=fx+5 ; t<80 ; t++ ) {
	    (void) move ( fy+i, t );
	    if ( ( check = inch() ) != ' ' ) {
		laserhit ( check, t, fy+i );
		break;
	    }
	    (void) addch ( ')' );
	}
    }
    update();
    for ( i=-1 ; i<=1 ; i++ ) {
	for ( t=fx+5 ; t<80 ; t++ ) {
	    (void) move ( fy+i, t );
	    check = inch();
	    if ( check != ')' ) break;
	    (void) addch ( ' ' );
	}
    }
    update();
    score -= 4;
}

/****************************************************************************\
 PROCEDURE: Blastwave.

\****************************************************************************/

int blastwave( fx, fy, flag )
    int fx, fy, flag;
{
    static int i, j, cmin, cmax, min, max, round;
    static char check;

    if ( flag ) {
	round = 0;
	min = fy-7;
	if ( min < 1 ) min = 1;
	max = fy+7;
	if ( max > 22 ) max = 22;

	cmin = fy;
	cmax = fy;
	i = fx+5;
	return ( 1 );
    } else {
	for ( j=cmin ; j<=cmax ; j++ ) {
	    vaddch ( i-1, j, ' ' );
	}
	if ( round % 2 == 1 ) {
	    if ( cmin > min )
		cmin--;
	    if ( cmax < max )
		cmax++;
	}
    }

    round++;

    for ( j=cmin ; j<=cmax ; j++ ) {
	(void) move ( j, i-4 );
	if ( ( check = inch() ) != ' ' ) laserhit ( check, i-4, j );
	(void) move ( j, i-3 );
	if ( ( check = inch() ) != ' ' ) laserhit ( check, i-3, j );
	(void) move ( j, i-2 );
	if ( ( check = inch() ) != ' ' ) laserhit ( check, i-2, j );
	(void) vaddch ( i, j, ')' );
    }

    if ( i++ > 79 )
	return ( 0 );
    else
	return ( 1 );
}

/****************************************************************************\
MAIN PROGRAM.  This is the messy bit that you can't stop fiddling with!

\****************************************************************************/

main( argc, argv )
    int argc;
    char *argv[];
{

/****************************************************************************\
VARIABLE DECLARATION
\****************************************************************************/

    int dead, i, j, keyflag, scoreopt, lasertype, blastflag;
    time_t big;
    char key;

/***************************************************************************\
|*									   *|
|*  Here's the redefine-keys bit, ripped out of tt.c.  S'a cinch - Mirk	   *|
|*									   *|
\***************************************************************************/

    {
      extern char *getenv ();	/* Declaration to keep lint happy. */
      char *cp;			/* Temporary pointer for getenv() */

      if ((cp = getenv ("VOMITKEYS")) != NULL) {
	if (*cp != '\0')
	  _up_key = *(cp++);
	if (*cp != '\0')
	  _down_key = *(cp++);
	if (*cp != '\0')
	  _left_key = *(cp++);
	if (*cp != '\0')
	  _right_key = *(cp++);
	if (*cp != '\0')
	  _fire_key = *(cp++);
	if (*cp != '\0')
	  _select_key = *(cp++);
	if (*cp != '\0')
	  _auto_destruct_key = *(cp++);
	if (*cp != '\0')
	  _cheat_key = *(cp++);
      }
    }

/****************************************************************************\
SETUP CONDITIONS FOR RUNNING
\****************************************************************************/

    if ( scoreopt = checkoption( argc, argv, "-s" ) ) {
	printscores ( argv[scoreopt] );
	exit ( 0 );
    }
    if ( checkoption ( argc, argv, "-levels" ) != 0 ) {
	for ( i=0; i<NUMSTAGES ; i++ ) {
	    (void) printf ( "%3i | %s \n" , i+1, stagename[i] );
	}
	exit ( 0 );
    }

    stage = testing(argc,argv);
    (void) initscr();
    (void) newwin ( 25, 80, 1, 1 );
    (void) cbreak();
    (void) noecho();
    (void) srandom ( time(&big) );
    if ( ! checkoption( argc, argv, "-f" ) ) introduction();

/****************************************************************************\
 SET PLAYING SCREEN
\****************************************************************************/

    (void) addstr ( ".0 " );
    (void) refresh();
    (void) move ( 0, 1 );
    standout();
    (void) printw (
"    ***  Chicken Vomit  *** By The Chief Slime Monster (And Max Abyt)    " );
    standend();
    (void) move ( 12, 5 );
    (void) addstr ( ship );
    printinfo ();
    update();


/****************************************************************************\
 VARIABLE INITIALISATION
\****************************************************************************/

    i = keyflag = shield = monsterstate1 = monsterstate2 = stageend = 0;
    wpoints = speedtime = lasertype = 0;
    y = 12;
    x = 5;
    dead = hit = hashcount = 0;

    stagechange();

/****************************************************************************\
 START OF CODE
\****************************************************************************/

    while ( dead != 1 ) {
	while ( hit == 0 ) {

	    switch ( pollkeyboard ( 0, 50000 ) ) {
	    case -1:
		(void) printw ( "Phlegm up! ERROR 1" );
		exit ( 1 );
		break;
	    case 0 :
		/* small delay, no input from keyboard */
		(void) usleep ( 10000 );
		keyflag = 0;
		break;
	    default:
		keyflag = 1;
		if ( read ( 0, &key, 1 ) != 1 ) {
		    (void) printw ( "Phlegm up! ERROR 2" );
		    exit ( 1 );
		}
		flush_keyboard();
	    }

	    if ( keyflag ) {
		if (key == _up_key) {
		    if ( y > 1 ) {
			hit = moveship ( x, y, 0, -1 );
			y--;
		    }
		} else if (key == _down_key) {
		    if ( y < 21 ) {
			hit = moveship ( x, y, 0, 1 );
			y++;
		    }
		} else if (key == _left_key) {
		    if ( speedtime == 0 ) {
			if ( x > 1 ) {
			    hit = moveship ( x, y, -2, 0 );
			    x -= 2;
			}
		    } else {
			speedtime--;
			if ( x > 3 ) {
			    hit = moveship ( x, y, -4, 0 );
			    x -= 4;
			}
		    }
		} else if (key == _right_key) {
		    if ( speedtime == 0 ) {
			if ( x < 70 ) {
			    hit = moveship ( x, y, 2, 0 );
			    x += 2;
			}
		    } else {
			speedtime--;
			if ( x < 68 ) {
			    hit = moveship ( x, y, 4, 0 );
			    x += 4;
			}
		    }
		} else if (key == _fire_key) {
		    switch ( lasertype ) {
		    case 0:
			laser ( x, y, 0 );
			break;
		    case 1:
			laser ( x, y, 1 );
			break;
		    default:
			hoopybeam ( x, y );
		    }
		} else if (key == _cheat_key) {
		    wpoints++;
		    update();
		} else if (key == _select_key) {
		    switch ( wpoints ) {
		    case 1:
			speedtime += NUMSPEEDMOVES;
			break;
		    case 2:
			bitbomb( x, y );
			break;
		    case 3:
			if ( shield < LIMSHIELD ) shield += 5;
			break;
		    case 4:
			lasertype = 1;
			ship[3] = ':';
			(void) move ( y, x );
			(void) addstr ( ship );
			break;
		    case 5:
			lasertype = 2;
			ship[3] = '{';
			(void) move ( y, x );
			(void) addstr( ship );
			break;
		    case 6:
			blastflag = blastwave ( x, y, 1 );
			break;
		    default:
			break;
		    }
		    wpoints = 0;
		} else if ( key == _auto_destruct_key ) {
		    (void) move ( 11, 20 );
		    (void) addstr ( "Auto destruct sequence initiated...." );
		    update();
		    (void) sleep ( 1 );
		    shield = 0;
		    hit = 1;
		} else
		    (void) usleep ( 10000 );
	    }
	    if (rnd() < HASHFREQ ) {
		i = (int) (1 + rnd() * .0070 );
		j = (int) (rnd() / 500 +1 );
		(void) move ( j, i );
		if ( inch() == ' ' ) (void) addch ( '#' );
	    }

	    printinfo();
 	    actmonster();
	    hit = hit + checkship();
	    if ( blastflag )
		blastflag = blastwave ( 0, 0, 0 );
	    update();
	    if ( stageend == 1 ) {
		stagefinish();
		hashcount = 0;
		blastflag = 0;
		stagechange();
	    }

	}
	/*  Ship is hit  */

	dead = 1;
	if ( shield != 0 ) {
	    shieldfire ( x, y );
	    shield--;
	    if ( shield == 0 ) {
		lasertype = 0;
		ship[3] = '-';
	    }
	    hit = dead = 0;
	}
    }

    die ( x, y );
    printinfo();
    flush_keyboard();
    dohiscore ( score, stage );
    if ( score < 0 ) {
	(void) move ( 13, 22 );
	(void) addstr ( "You absolute phlegm-head!" );
    }
    (void) move ( 23, 1 );
    (void) addstr ( "Back to sanity..." );
    (void) refresh();
    (void) nocbreak();
    (void) endwin();
}
